<?php

// Dans un controller

namespace App\Controller;

use App\Controller\AppController;
use Cake\Core\Configure;
use Cake\Utility\Hash;

/**
 * CakePHP UsersController
 * @author mpastor
 */
class SoftwaresController extends AppController
{

    public function initialize()
    {
        parent::initialize();
        $this->loadComponent('ComptoirServer');
    }

    public function isAuthorized($user)
    {
        if ($this->Auth->identify()){
            return true;
        }

        return parent::isAuthorized($user); // TODO: Change the autogenerated stub
    }

    public function beforeFilter(\Cake\Event\Event $event)
    {
        $this->Auth->allow([
            'servicesProviders',
            'usersSoftware',
            'alternativeTo',
            'workswellSoftware',
            'backersSoftware',
            'screenshots',
            'reviewsSoftware',
            'formula',
        ]);
        parent::beforeFilter($event);

    }

    public function view($id = null)
    {

        $result = $this->ComptoirServer->get(COMPTOIR_SRV_API . "softwares/" . $id, 'software', 'software');
        $software = Hash::get((array)$result, "software");

        //For Social MEDIAS => OPENGRAPH
        $openGraph = [
            "title"  => $software->softwarename,
            "description" => $software->description,
            "image" => COMPTOIR_SRV_URL . $software->logo_directory . DS . $software->photo,
            "imageSize" => ['width' => '200', 'height' => '300']
        ];

        $card = [
            "title"  => $software->softwarename,
            "description" => $software->description,
            "image" => COMPTOIR_SRV_URL . $software->logo_directory . DS . $software->photo,
            "imageSize" => ['width' => '200', 'height' => '300']
        ];

        $this->set('openGraph', $openGraph);
        $this->set('card', $card);

    }

    public function add()
    {

        if (!empty($this->request->data)){
            $result = $this->ComptoirServer->post(COMPTOIR_SRV_API . "softwares/", 'software', 'software',$this->request->data);
        }else{
            $result = $this->ComptoirServer->post(COMPTOIR_SRV_API . "softwares/", 'licenses', 'licenses',[]);
        }


        $this->set("rules", Hash::get((array)$result, "rules"));
        $this->set("licenses", Hash::get((array)$result, "licenses"));
        $this->set("software", Hash::get((array)$result, "software"));
        $this->set("message", Hash::get((array)$result, "message"));

        if (Hash::get((array)$result, "message") === "Success"){
            return  $this->Flash->success(__d("Forms","Congratulations ! Thanks to you a new software is on the Comptoir du Libre !"));
        }
        elseif (Hash::get((array)$result, "message") === "Error"){
            return $this->Flash->error(__d("Forms","Your request to add a software failed, please follow rules in red."));
        }
    }


    public function edit($id = null)
    {

        if (!empty($this->request->data)){
            $result = $this->ComptoirServer->put(COMPTOIR_SRV_API . "softwares/edit/". $id, 'software', 'software',$this->request->data);
        }else{
            $result = $this->ComptoirServer->get(COMPTOIR_SRV_API . "softwares/edit/". $id, 'licenses', 'licenses',[]);

        }

        $this->set("rules", Hash::get((array)$result, "rules"));
        $this->set("licenses", Hash::get((array)$result, "licenses"));
        $this->set("software", Hash::get((array)$result, "software"));
        $this->set("message", Hash::get((array)$result, "message"));
        $this->set("errors", Hash::get((array)$result, "errors"));
        $this->set("id",$id);
        if (Hash::get((array)$result, "message") === "Success"){
            return  $this->Flash->success(__d("Forms","Edit.Congratulations.software.success"));
        }
        elseif (Hash::get((array)$result, "message") === "Error"){
            return $this->Flash->error(__d("Forms","Edit.Congratulations.software.error"));
        }
    }


    public function index()
    {

        if (!empty($this->request->data)) {
            $sort=explode(".",$this->request->data["order"]);
            $this->request->data["sort"] = $sort[0];
            $this->request->data["direction"] = $sort[1];
            $this->request->url = str_replace(["en/", "fr/"], "", $this->request->url);
            $result = $this->ComptoirServer->get(COMPTOIR_SRV_API . $this->request->url, 'softwares', 'softwares',$this->request->data);
        }else{
            $this->ComptoirServer->get(COMPTOIR_SRV_API . "softwares/", 'softwares', 'softwares');
        }

        $reviewed = [
            "false" => __d("Forms","Search.Filter.NoFilterOnReview"),
            "true"=>__d("Forms","Search.Filter.WithReview"),
        ];

        $screenCaptured = [
            "false" => __d("Forms","Search.Filter.NoFilterOnScreenshot"),
            "true"=>__d("Forms","Search.Filter.WithScreenshot"),
        ];

        $used = [
            "false" => __d("Forms","Search.Filter.NoFilterOnUser"),
            "true"=>__d("Forms","Search.Filter.WithUser"),
        ];

        $hasServiceProvider = [
            "false" => __d("Forms","Search.Filter.NoFilterOnServiceProvider"),
            "true"=>__d("Forms","Search.Filter.ServiceProvider"),
        ];

        $order = [
            "softwarename.asc"=>__d("Forms","Search.Sort.softwarenameAsc"),
            "softwarename.desc"=>__d("Forms","Search.Sort.softwarenameDesc"),
            "created.asc" => __d("Forms","Search.Sort.createdAsc"),
            "created.desc" => __d("Forms","Search.Sort.createdDesc"),
            "modified.asc"=>__d("Forms","Search.Sort.modifiedAsc"),
            "modified.desc"=>__d("Forms","Search.Sort.modifiedDesc"),
            "average_review.asc"=>__d("Forms","Search.Sort.averageReviewsAsc"),
            "average_review.desc"=>__d("Forms","Search.Sort.averageReviewsDesc"),
        ];

        $this->set('reviewed',$reviewed);
        $this->set('used',$used);
        $this->set('hasServiceProvider',$hasServiceProvider);
        $this->set('screenCaptured',$screenCaptured);
        $this->set('order',$order);
    }

    public function servicesProviders($id = null)
    {
        $this->ComptoirServer->get(COMPTOIR_SRV_API . "softwares/servicesProviders/" . $id, 'software', 'software');
    }

    public function usersSoftware($id = null)
    {
        $this->ComptoirServer->get(COMPTOIR_SRV_API . "softwares/usersSoftware/" . $id, 'software', 'software');
    }

    /**
     * Fall back the current user from the users' list of the current software
     * @param integer $id
     * @return \Cake\Network\Response|null
     */
    public function fallBackusersSoftware ($id = null) {
        $result = $this->ComptoirServer->delete(COMPTOIR_SRV_API . "softwares/deleteUsersSoftware/" . $id, 'software', 'software');

        if (Hash::get((array)$result, "message") === "Success"){
            $this->Flash->success(__d("Forms","Your are not know as user of anymore."));

        }
        elseif (Hash::get((array)$result, "message") === "Error"){
            $this->Flash->error(__d("Forms","We can not respond to your request"));
        }
        return $this->redirect(["controller"=>"Softwares","action"=>"view", "language"=>$this->request->param("language"),$id]);
    }

    /**
     * Fall back the current connected user form the users' list of the current software
     * @param null $id
     * @return \Cake\Network\Response|null
     */
    public function fallBackServicesProvider ($id = null) {
        $result = $this->ComptoirServer->delete(COMPTOIR_SRV_API . "softwares/deleteServicesProviders/" . $id, 'software', 'software');

        if (Hash::get((array)$result, "message") === "Success"){
            $this->Flash->success(__d("Forms","Your are not know as service provider of anymore."));

        }
        elseif (Hash::get((array)$result, "message") === "Error"){
            $this->Flash->error(__d("Forms","We can not respond to your request"));
        }
        return $this->redirect(["controller"=>"Softwares","action"=>"view", "language"=>$this->request->param("language"),$id]);

    }

    public function addUserOf ($id = null )
    {
        $result = $this->ComptoirServer->post(COMPTOIR_SRV_API ."softwares/usersSoftware/" . $id , 'software', 'software',$this->Auth->user('id'));

        if (Hash::get((array)$result, "message") === "Success"){
            $this->Flash->success(__d("Forms","Your are now known as user of."));

        }
        elseif (Hash::get((array)$result, "message") === "Error"){

            ($result->software->software_id->_isUnique != null)
                ? $this->Flash->error(__d("Forms","You can not be declared twice as user of {0}.",$result->softwareName))
                : $this->Flash->error(__d("Forms","We can not respond to your request"));

        }
        return $this->redirect(["controller"=>"Softwares","action"=>"view", "language"=>$this->request->param("language"),$id]);
    }

    public function addServicesProvider ($id = null )
    {
        $this->viewBuilder()->template("view");
        $result = $this->ComptoirServer->post(COMPTOIR_SRV_API ."softwares/servicesProviders/" . $id , 'software', 'software',$this->Auth->user('id'));
        if (Hash::get((array)$result, "message") === "Success"){
            $this->Flash->success(__d("Forms","Your are now known as service providers for {0}.",$result->software->softwarename));

        }
        elseif (Hash::get((array)$result, "message") === "Error"){
            ($result->software->software_id->_isUnique != null)
                ? $this->Flash->error(__d("Forms","You can not be declared twice as service provider for {0}.",$result->softwareName))
                : $this->Flash->error(__d("Forms","We can not respond to your request."));

        }
        return $this->redirect(["controller"=>"Softwares","action"=>"view", "language"=>$this->request->param("language"),$id]);

    }

    public function alternativeTo($id = null)
    {
        $this->ComptoirServer->get(COMPTOIR_SRV_API . "softwares/alternativeTo/" . $id, 'software', 'software');
    }

    public function workswellSoftware($id = null)
    {

        $this->ComptoirServer->get(COMPTOIR_SRV_API . "softwares/workswellSoftwares/" . $id, 'software', 'software');
    }

    public function backersSoftware($id = null)
    {

        $this->ComptoirServer->get(COMPTOIR_SRV_API . "softwares/backersSoftware/" . $id, 'software', 'software');
    }

    public function screenshots($id = null)
    {
        $this->ComptoirServer->get(COMPTOIR_SRV_API . "softwares/" . $id, 'software', 'software');

        $this->ComptoirServer->get(COMPTOIR_SRV_API . "softwares/" . $id ."/screenshots", 'screenshots', 'screenshots');

    }

    public function reviewsSoftware($id = null)
    {
        $this->ComptoirServer->get(COMPTOIR_SRV_API . "softwares/" . $id, 'software', 'software');

        $this->ComptoirServer->get(COMPTOIR_SRV_API . "softwares/" . $id ."/reviews", 'reviews', 'reviews');
    }

    /**
    * Sm means SourceMetric, objective sources of metric,
    * Cm means ComptoirMetric, metrics from Comptoir du libre .
    */
    public function formula() {
        $this->set('formula', ["metrics" => ["Sm_last_commit_age" => __d("ProjectController", "The age(in days) of the last commit for the project."),
         "Sm_project_age" => __d("ProjectController", "The age of the project (in days)"),
        "Sm_delta_commit_one_month" => __d("ProjectController", "The difference between the number of commits in a month and the average of the number of commits in one month for all the projects."),
        "Sm_delta_commit_twelve_month" => __d("ProjectController", "The difference between the number of commits in twelve months and the average of the number of commits in twelve months for all the projects."),
        "Sm_number_of_contributors" => __d("ProjectController", "The number of contributors working on the project."), "Sm_high_committer_percent" => __d("ProjectController", "The contribution percentage of the highest contributor."),
        "Cm_code_gouv_label" => __d("ProjectController", "The project obtains the label code gov if it is available on code.gouv.fr"),
        "Cm_declared_users" => __d("ProjectController", "The number of declared users on Comptoir du libre."),
        "Cm_average_review_score" => __d("ProjectController", "The average review score for this project on Comptoir du libre."),
        "Cm_screenshots" => __d("ProjectController", "The number of screenshots available on Comptoir du libre.")
        ],
        "nbSource" => [ sizeof(Configure::read("SOURCE_CODE_METRICS"))]
        ]);
    }

    /**
     * Post a review to the comptoir-srv
     */
    public function addReview ($id = null) {

        if (!empty($this->request->data)){
            $result = $this->ComptoirServer->post(COMPTOIR_SRV_API . "softwares/".$this->request->data['software_id']."/reviews" , 'software', 'software',$this->request->data);

            if (Hash::get((array)$result, "message") === "Success"){
                $this->Flash->success(__d("Forms","Your review has been posted"));
                return $this->redirect(["controller"=>"Softwares","action"=>"view", "language"=>$this->request->param("language"),$this->request->data['software_id']]);
            }
            elseif (Hash::get((array)$result, "message") === "Error"){

                (isset ($result->review->software_id->_isUnique) )
                    ? $this->Flash->error(__d("Forms",$result->review->software_id->_isUnique))
                    : $this->Flash->error(__d("Forms","Your post failed, please follow rules in red."));
            }
            elseif (Hash::get((array)$result, "code") === 403) {
                $this->Flash->error(__d("Forms","Softwares.Review.add.error.notAuthorized"));
                return $this->redirect(["controller"=>"Softwares","action"=>"view", "language"=>$this->request->param("language"),$this->request->data['software_id']]);
            }

        }
            $userId = $this->Auth->user('id');
            $softwareId = $id ;
            $this->set(compact('review', 'userId', 'softwareId'));
            $this->set('_serialize', ['review', 'userId', 'softwareId']);

    }

    public function addScreenshots ($id = null) {

    }
}
